unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, SecureCode, StdCtrls;

type
  TMainForm = class(TForm)
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    edName: TEdit;
    edDecryptKey: TEdit;
    btCheck: TButton;
    Label3: TLabel;
    edLicense: TEdit;
    memLog: TMemo;
    btLoadBlock: TButton;
    Button2: TButton;
    btDebugger: TButton;
    Button1: TButton;
    procedure btCheckClick(Sender: TObject);
    procedure edNameChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure btLoadBlockClick(Sender: TObject);
    procedure btDebuggerClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Dclarations prives }
    FSecureCode : TSecureCode;
    procedure GetWatermarkInfo;
    procedure CheckLicense;
    procedure FileModified;
    function GenerateKey(fromName : string) : string;
  public
    { Dclarations publiques }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

procedure TMainForm.GetWatermarkInfo;
label
  scBlockStart; // Require if you wish to control block individually
var
  scBlockID : Cardinal; // Require if you wish to control block individually
begin
  {$I SC_INIT.INC} // Require to secure block individually
  if FSecureCode.Decrypt(scBlockID) then
  begin
    {$I SC_START_IDV.INC} // Require to secure block individually
    if FSecureCode.Watermark <> '' then
    begin
      {$I WATERMARK.INC}
      memLog.Lines.Add('Watermark is ' + FSecureCode.Watermark)
    end
    else
      memLog.Lines.Add('None watermark found');
    {$I SC_END.INC}
    FSecureCode.SecureBlock(scBlockID);
  end;
end;

(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

procedure TMainForm.btLoadBlockClick(Sender: TObject);
begin
  with TOpenDialog.Create(nil) do
  begin
    Filter := 'SecureCode Block(s) code *.bin | *.bin';
    Title  := 'Select SecureCode blocks code file *.bin ...';
    if Execute then
       FSecureCode.LoadBlockFromFile(Filename);
    Free;
  end;
end;

(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

procedure TMainForm.btDebuggerClick(Sender: TObject);
begin
  // Using this command you can customize and detect yourself when your function is traced.
  if not FSecureCode.IsDebugging then
    memLog.Lines.Add('No debugger, it is ok.')
  else
    // If you are under Delphi, you'll see this message, because Delphi debugger is running
    memLog.Lines.Add('Debugger has been detected!');
end;

(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

procedure TMainForm.Button1Click(Sender: TObject);
begin
  edLicense.Text := 'seewm';
  btCheck.OnClick(nil);
end;

procedure TMainForm.Button2Click(Sender: TObject);
begin
  Application.Terminate;
end;

(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

procedure TMainForm.CheckLicense;
label
  scBlockStart; // Require to secure block individually
var
  scBlockID : Cardinal; // Require to secure block individually
begin
  try
    with FSecureCode do
    begin
      if not IsDebugging then // Check debugger before to continue
      begin
        Key := edDecryptKey.Text;
        {$I SC_INIT.INC} // Require to secure block individually
        if Decrypt(scBlockID) then
        begin
          {$I SC_START_IDV.INC} // Require to secure block individually
            if edLicense.Text = GenerateKey(edName.Text) then
            begin
              {$I WATERMARK.INC} // Hide watermark information between your code
              memLog.Lines.Add('License valid, thank you')
            end
            else
              memLog.Lines.Add('Sorry, such license is invalid!');

          {$I SC_END.INC}
          FSecureCode.SecureBlock(scBlockID);  // Re-secure source code after execution
        end;
      end;
    end;
  except on e:exception do
    memLog.Lines.Add('Error : ' + e.Message);
  end;
end;

(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

procedure TMainForm.FileModified;
begin
  ShowMessage('This executable file has been modified!, please restore from original setup, thank you');
  memLog.Lines.Add('Executable file corrupted, please restore from original...');
  // here you can choose to close application or whateven you want...
end;

(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

procedure TMainForm.edNameChange(Sender: TObject);
begin
  edLicense.Hint := 'Your license for your name is : ' + GenerateKey(edName.Text);
  if not edLicense.ShowHint then
    edLicense.ShowHint := True;
end;

(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FSecureCode := TSecureCode.Create;
  FSecureCode.FileModified := FileModified;
  FSecureCode.Key := 'demo';
  FSecureCode.Init;
end;

(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  if assigned(FSecureCode) then
    FSecureCode.Free;
end;

(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

function TMainForm.GenerateKey(fromName : string) : string;
const
  CHARS = '123456789';
label
  scBlockStart; // Require to secure block individually
var
  i : Cardinal;
  Rnd : Cardinal;
  tmp : string;
  scBlockID : Cardinal; // Require to secure block individually
begin
  try
    {$I SC_INIT.INC}
    if FSecureCode.Decrypt(scBlockID) then
    begin
      {$I SC_START_IDV.INC}
        tmp := '';
        Rnd := 0;
        RandSeed := Length(fromName) * 999;
        for i := 1 to 8 do
        begin
          repeat
            Rnd := Random(Length(CHARS));
          until Rnd <> 0;
          tmp := tmp + CHARS[Rnd];
        end;
        Insert('-',tmp,5);
        result := Tmp;
      {$I SC_END.INC}
      FSecureCode.SecureBlock(scBlockID); // SecureCode protect block after execution
    end;
  except on e:exception do
    memLog.Lines.Add('Error, ' + e.Message);
  end;
end;

(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

procedure TMainForm.btCheckClick(Sender: TObject);
begin
  if edLicense.Text = 'seewm' then
    GetWatermarkInfo
  else
    CheckLicense;
end;

(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)


end.
